"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const compiler = require("../compiler");
const typescript_1 = require("../typescript");
// when changing this, make sure to run `npm run code-generate`.
// that will automatically update all other parts of the application that need to be updated when this changes.
// using an "unknown" type here because I couldn't figure out a way of getting the typescript compiler to understand this
exports.kindToWrapperMappings = {
    [typescript_1.SyntaxKind.SourceFile]: compiler.SourceFile,
    [typescript_1.SyntaxKind.ArrayBindingPattern]: compiler.ArrayBindingPattern,
    [typescript_1.SyntaxKind.ArrayLiteralExpression]: compiler.ArrayLiteralExpression,
    [typescript_1.SyntaxKind.ArrayType]: compiler.ArrayTypeNode,
    [typescript_1.SyntaxKind.ArrowFunction]: compiler.ArrowFunction,
    [typescript_1.SyntaxKind.AsExpression]: compiler.AsExpression,
    [typescript_1.SyntaxKind.AwaitExpression]: compiler.AwaitExpression,
    [typescript_1.SyntaxKind.BindingElement]: compiler.BindingElement,
    [typescript_1.SyntaxKind.BinaryExpression]: compiler.BinaryExpression,
    [typescript_1.SyntaxKind.Block]: compiler.Block,
    [typescript_1.SyntaxKind.BreakStatement]: compiler.BreakStatement,
    [typescript_1.SyntaxKind.CallExpression]: compiler.CallExpression,
    [typescript_1.SyntaxKind.CallSignature]: compiler.CallSignatureDeclaration,
    [typescript_1.SyntaxKind.CaseBlock]: compiler.CaseBlock,
    [typescript_1.SyntaxKind.CaseClause]: compiler.CaseClause,
    [typescript_1.SyntaxKind.CatchClause]: compiler.CatchClause,
    [typescript_1.SyntaxKind.ClassDeclaration]: compiler.ClassDeclaration,
    [typescript_1.SyntaxKind.ClassExpression]: compiler.ClassExpression,
    [typescript_1.SyntaxKind.ConditionalType]: compiler.ConditionalTypeNode,
    [typescript_1.SyntaxKind.Constructor]: compiler.ConstructorDeclaration,
    [typescript_1.SyntaxKind.ConstructorType]: compiler.ConstructorTypeNode,
    [typescript_1.SyntaxKind.ConstructSignature]: compiler.ConstructSignatureDeclaration,
    [typescript_1.SyntaxKind.ContinueStatement]: compiler.ContinueStatement,
    [typescript_1.SyntaxKind.CommaListExpression]: compiler.CommaListExpression,
    [typescript_1.SyntaxKind.ComputedPropertyName]: compiler.ComputedPropertyName,
    [typescript_1.SyntaxKind.ConditionalExpression]: compiler.ConditionalExpression,
    [typescript_1.SyntaxKind.DebuggerStatement]: compiler.DebuggerStatement,
    [typescript_1.SyntaxKind.Decorator]: compiler.Decorator,
    [typescript_1.SyntaxKind.DefaultClause]: compiler.DefaultClause,
    [typescript_1.SyntaxKind.DeleteExpression]: compiler.DeleteExpression,
    [typescript_1.SyntaxKind.DoStatement]: compiler.DoStatement,
    [typescript_1.SyntaxKind.ElementAccessExpression]: compiler.ElementAccessExpression,
    [typescript_1.SyntaxKind.EmptyStatement]: compiler.EmptyStatement,
    [typescript_1.SyntaxKind.EnumDeclaration]: compiler.EnumDeclaration,
    [typescript_1.SyntaxKind.EnumMember]: compiler.EnumMember,
    [typescript_1.SyntaxKind.ExportAssignment]: compiler.ExportAssignment,
    [typescript_1.SyntaxKind.ExportDeclaration]: compiler.ExportDeclaration,
    [typescript_1.SyntaxKind.ExportSpecifier]: compiler.ExportSpecifier,
    [typescript_1.SyntaxKind.ExpressionWithTypeArguments]: compiler.ExpressionWithTypeArguments,
    [typescript_1.SyntaxKind.ExpressionStatement]: compiler.ExpressionStatement,
    [typescript_1.SyntaxKind.ExternalModuleReference]: compiler.ExternalModuleReference,
    [typescript_1.SyntaxKind.QualifiedName]: compiler.QualifiedName,
    [typescript_1.SyntaxKind.ForInStatement]: compiler.ForInStatement,
    [typescript_1.SyntaxKind.ForOfStatement]: compiler.ForOfStatement,
    [typescript_1.SyntaxKind.ForStatement]: compiler.ForStatement,
    [typescript_1.SyntaxKind.FunctionDeclaration]: compiler.FunctionDeclaration,
    [typescript_1.SyntaxKind.FunctionExpression]: compiler.FunctionExpression,
    [typescript_1.SyntaxKind.FunctionType]: compiler.FunctionTypeNode,
    [typescript_1.SyntaxKind.GetAccessor]: compiler.GetAccessorDeclaration,
    [typescript_1.SyntaxKind.HeritageClause]: compiler.HeritageClause,
    [typescript_1.SyntaxKind.Identifier]: compiler.Identifier,
    [typescript_1.SyntaxKind.IfStatement]: compiler.IfStatement,
    [typescript_1.SyntaxKind.ImportClause]: compiler.ImportClause,
    [typescript_1.SyntaxKind.ImportDeclaration]: compiler.ImportDeclaration,
    [typescript_1.SyntaxKind.ImportEqualsDeclaration]: compiler.ImportEqualsDeclaration,
    [typescript_1.SyntaxKind.ImportSpecifier]: compiler.ImportSpecifier,
    [typescript_1.SyntaxKind.ImportType]: compiler.ImportTypeNode,
    [typescript_1.SyntaxKind.IndexedAccessType]: compiler.IndexedAccessTypeNode,
    [typescript_1.SyntaxKind.IndexSignature]: compiler.IndexSignatureDeclaration,
    [typescript_1.SyntaxKind.InferType]: compiler.InferTypeNode,
    [typescript_1.SyntaxKind.InterfaceDeclaration]: compiler.InterfaceDeclaration,
    [typescript_1.SyntaxKind.IntersectionType]: compiler.IntersectionTypeNode,
    [typescript_1.SyntaxKind.JSDocAugmentsTag]: compiler.JSDocAugmentsTag,
    [typescript_1.SyntaxKind.JSDocClassTag]: compiler.JSDocClassTag,
    [typescript_1.SyntaxKind.JSDocFunctionType]: compiler.JSDocFunctionType,
    [typescript_1.SyntaxKind.JSDocReturnTag]: compiler.JSDocReturnTag,
    [typescript_1.SyntaxKind.JSDocSignature]: compiler.JSDocSignature,
    [typescript_1.SyntaxKind.JSDocTag]: compiler.JSDocUnknownTag,
    [typescript_1.SyntaxKind.JSDocTypeExpression]: compiler.JSDocTypeExpression,
    [typescript_1.SyntaxKind.JSDocTypeTag]: compiler.JSDocTypeTag,
    [typescript_1.SyntaxKind.JSDocTypedefTag]: compiler.JSDocTypedefTag,
    [typescript_1.SyntaxKind.JSDocParameterTag]: compiler.JSDocParameterTag,
    [typescript_1.SyntaxKind.JSDocPropertyTag]: compiler.JSDocPropertyTag,
    [typescript_1.SyntaxKind.JsxAttribute]: compiler.JsxAttribute,
    [typescript_1.SyntaxKind.JsxClosingElement]: compiler.JsxClosingElement,
    [typescript_1.SyntaxKind.JsxClosingFragment]: compiler.JsxClosingFragment,
    [typescript_1.SyntaxKind.JsxElement]: compiler.JsxElement,
    [typescript_1.SyntaxKind.JsxExpression]: compiler.JsxExpression,
    [typescript_1.SyntaxKind.JsxFragment]: compiler.JsxFragment,
    [typescript_1.SyntaxKind.JsxOpeningElement]: compiler.JsxOpeningElement,
    [typescript_1.SyntaxKind.JsxOpeningFragment]: compiler.JsxOpeningFragment,
    [typescript_1.SyntaxKind.JsxSelfClosingElement]: compiler.JsxSelfClosingElement,
    [typescript_1.SyntaxKind.JsxSpreadAttribute]: compiler.JsxSpreadAttribute,
    [typescript_1.SyntaxKind.JsxText]: compiler.JsxText,
    [typescript_1.SyntaxKind.LabeledStatement]: compiler.LabeledStatement,
    [typescript_1.SyntaxKind.LiteralType]: compiler.LiteralTypeNode,
    [typescript_1.SyntaxKind.MetaProperty]: compiler.MetaProperty,
    [typescript_1.SyntaxKind.MethodDeclaration]: compiler.MethodDeclaration,
    [typescript_1.SyntaxKind.MethodSignature]: compiler.MethodSignature,
    [typescript_1.SyntaxKind.ModuleBlock]: compiler.ModuleBlock,
    [typescript_1.SyntaxKind.ModuleDeclaration]: compiler.NamespaceDeclaration,
    [typescript_1.SyntaxKind.NamedExports]: compiler.NamedExports,
    [typescript_1.SyntaxKind.NamedImports]: compiler.NamedImports,
    [typescript_1.SyntaxKind.NamespaceImport]: compiler.NamespaceImport,
    [typescript_1.SyntaxKind.NewExpression]: compiler.NewExpression,
    [typescript_1.SyntaxKind.NonNullExpression]: compiler.NonNullExpression,
    [typescript_1.SyntaxKind.NotEmittedStatement]: compiler.NotEmittedStatement,
    [typescript_1.SyntaxKind.NoSubstitutionTemplateLiteral]: compiler.NoSubstitutionTemplateLiteral,
    [typescript_1.SyntaxKind.NumericLiteral]: compiler.NumericLiteral,
    [typescript_1.SyntaxKind.ObjectBindingPattern]: compiler.ObjectBindingPattern,
    [typescript_1.SyntaxKind.ObjectLiteralExpression]: compiler.ObjectLiteralExpression,
    [typescript_1.SyntaxKind.OmittedExpression]: compiler.OmittedExpression,
    [typescript_1.SyntaxKind.Parameter]: compiler.ParameterDeclaration,
    [typescript_1.SyntaxKind.ParenthesizedExpression]: compiler.ParenthesizedExpression,
    [typescript_1.SyntaxKind.ParenthesizedType]: compiler.ParenthesizedTypeNode,
    [typescript_1.SyntaxKind.PartiallyEmittedExpression]: compiler.PartiallyEmittedExpression,
    [typescript_1.SyntaxKind.PostfixUnaryExpression]: compiler.PostfixUnaryExpression,
    [typescript_1.SyntaxKind.PrefixUnaryExpression]: compiler.PrefixUnaryExpression,
    [typescript_1.SyntaxKind.PropertyAccessExpression]: compiler.PropertyAccessExpression,
    [typescript_1.SyntaxKind.PropertyAssignment]: compiler.PropertyAssignment,
    [typescript_1.SyntaxKind.PropertyDeclaration]: compiler.PropertyDeclaration,
    [typescript_1.SyntaxKind.PropertySignature]: compiler.PropertySignature,
    [typescript_1.SyntaxKind.RegularExpressionLiteral]: compiler.RegularExpressionLiteral,
    [typescript_1.SyntaxKind.ReturnStatement]: compiler.ReturnStatement,
    [typescript_1.SyntaxKind.SetAccessor]: compiler.SetAccessorDeclaration,
    [typescript_1.SyntaxKind.ShorthandPropertyAssignment]: compiler.ShorthandPropertyAssignment,
    [typescript_1.SyntaxKind.SpreadAssignment]: compiler.SpreadAssignment,
    [typescript_1.SyntaxKind.SpreadElement]: compiler.SpreadElement,
    [typescript_1.SyntaxKind.StringLiteral]: compiler.StringLiteral,
    [typescript_1.SyntaxKind.SwitchStatement]: compiler.SwitchStatement,
    [typescript_1.SyntaxKind.SyntaxList]: compiler.SyntaxList,
    [typescript_1.SyntaxKind.TaggedTemplateExpression]: compiler.TaggedTemplateExpression,
    [typescript_1.SyntaxKind.TemplateExpression]: compiler.TemplateExpression,
    [typescript_1.SyntaxKind.TemplateHead]: compiler.TemplateHead,
    [typescript_1.SyntaxKind.TemplateMiddle]: compiler.TemplateMiddle,
    [typescript_1.SyntaxKind.TemplateSpan]: compiler.TemplateSpan,
    [typescript_1.SyntaxKind.TemplateTail]: compiler.TemplateTail,
    [typescript_1.SyntaxKind.ThisType]: compiler.ThisTypeNode,
    [typescript_1.SyntaxKind.ThrowStatement]: compiler.ThrowStatement,
    [typescript_1.SyntaxKind.TryStatement]: compiler.TryStatement,
    [typescript_1.SyntaxKind.TupleType]: compiler.TupleTypeNode,
    [typescript_1.SyntaxKind.TypeAliasDeclaration]: compiler.TypeAliasDeclaration,
    [typescript_1.SyntaxKind.TypeAssertionExpression]: compiler.TypeAssertion,
    [typescript_1.SyntaxKind.TypeLiteral]: compiler.TypeLiteralNode,
    [typescript_1.SyntaxKind.TypeParameter]: compiler.TypeParameterDeclaration,
    [typescript_1.SyntaxKind.TypeReference]: compiler.TypeReferenceNode,
    [typescript_1.SyntaxKind.UnionType]: compiler.UnionTypeNode,
    [typescript_1.SyntaxKind.VariableDeclaration]: compiler.VariableDeclaration,
    [typescript_1.SyntaxKind.VariableDeclarationList]: compiler.VariableDeclarationList,
    [typescript_1.SyntaxKind.VariableStatement]: compiler.VariableStatement,
    [typescript_1.SyntaxKind.JSDocComment]: compiler.JSDoc,
    [typescript_1.SyntaxKind.TypePredicate]: compiler.TypeNode,
    [typescript_1.SyntaxKind.SemicolonToken]: compiler.Node,
    [typescript_1.SyntaxKind.TypeOfExpression]: compiler.TypeOfExpression,
    [typescript_1.SyntaxKind.WhileStatement]: compiler.WhileStatement,
    [typescript_1.SyntaxKind.WithStatement]: compiler.WithStatement,
    [typescript_1.SyntaxKind.YieldExpression]: compiler.YieldExpression,
    // keywords
    [typescript_1.SyntaxKind.AnyKeyword]: compiler.Expression,
    [typescript_1.SyntaxKind.BooleanKeyword]: compiler.Expression,
    [typescript_1.SyntaxKind.FalseKeyword]: compiler.BooleanLiteral,
    [typescript_1.SyntaxKind.ImportKeyword]: compiler.ImportExpression,
    [typescript_1.SyntaxKind.InferKeyword]: compiler.Node,
    [typescript_1.SyntaxKind.NeverKeyword]: compiler.Expression,
    [typescript_1.SyntaxKind.NullKeyword]: compiler.NullLiteral,
    [typescript_1.SyntaxKind.NumberKeyword]: compiler.Expression,
    [typescript_1.SyntaxKind.ObjectKeyword]: compiler.Expression,
    [typescript_1.SyntaxKind.StringKeyword]: compiler.Expression,
    [typescript_1.SyntaxKind.SymbolKeyword]: compiler.Expression,
    [typescript_1.SyntaxKind.SuperKeyword]: compiler.SuperExpression,
    [typescript_1.SyntaxKind.ThisKeyword]: compiler.ThisExpression,
    [typescript_1.SyntaxKind.TrueKeyword]: compiler.BooleanLiteral,
    [typescript_1.SyntaxKind.UndefinedKeyword]: compiler.Expression,
    [typescript_1.SyntaxKind.VoidExpression]: compiler.VoidExpression
};
