"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const errors = require("../../errors");
const typescript_1 = require("../../typescript");
const utils_1 = require("../../utils");
/**
 * Gets a range of the array.
 * @remarks NOTICE: This will ignore comments in the array.
 */
function getRangeWithoutCommentsFromArray(array, index, length, expectedKind) {
    const children = [];
    while (index < array.length && children.length < length) {
        const child = array[index];
        const childKind = child.getKind();
        // ignore any comments that may have been on the structure's leading trivia or trailing trivia
        if (childKind !== typescript_1.SyntaxKind.SingleLineCommentTrivia && childKind !== typescript_1.SyntaxKind.MultiLineCommentTrivia) {
            if (childKind !== expectedKind) {
                throw new errors.NotImplementedError(`Unexpected! Inserting syntax kind of ${utils_1.getSyntaxKindName(expectedKind)}` +
                    `, but ${child.getKindName()} was inserted.`);
            }
            children.push(child);
        }
        index++;
    }
    if (children.length !== length)
        throw new errors.NotImplementedError(`Unexpected! Inserted ${length} child/children, but ${children.length} were inserted.`);
    return children;
}
exports.getRangeWithoutCommentsFromArray = getRangeWithoutCommentsFromArray;
