"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const errors = require("../../errors");
const utils_1 = require("../../utils");
/**
 * Attempts to use a node handler, but if it fails it will forget all the nodes' children.
 */
class TryOrForgetNodeHandler {
    constructor(handler) {
        this.handler = handler;
    }
    handleNode(currentNode, newNode, newSourceFile) {
        /* istanbul ignore next */
        if (!utils_1.TypeGuards.isSourceFile(currentNode))
            throw new errors.InvalidOperationError(`Can only use a ${"TryOrForgetNodeHandler"} with a source file.`);
        try {
            this.handler.handleNode(currentNode, newNode, newSourceFile);
        }
        catch (ex) {
            currentNode._context.logger.warn("Could not replace tree, so forgetting all nodes instead. Message: " + ex);
            // forget all the source file's nodes
            currentNode.getChildSyntaxListOrThrow().forget();
            // replace the source file with the temporary source file
            currentNode._context.compilerFactory.replaceCompilerNode(currentNode, newNode);
        }
    }
}
exports.TryOrForgetNodeHandler = TryOrForgetNodeHandler;
