"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const utils_1 = require("../utils");
class SettingsContainer {
    /** @private */
    constructor(defaultSettings) {
        this._defaultSettings = utils_1.ObjectUtils.assign({}, defaultSettings);
        this._settings = defaultSettings;
    }
    /**
     * Resets the settings to the default.
     */
    reset() {
        this._settings = utils_1.ObjectUtils.assign({}, this._defaultSettings);
        this._fireModified();
    }
    /**
     * Gets a copy of the settings as an object.
     */
    get() {
        return utils_1.ObjectUtils.assign({}, this._settings);
    }
    /**
     * Sets one or all of the settings.
     * @param settings - Settings to set.
     */
    set(settings) {
        utils_1.ObjectUtils.assign(this._settings, settings);
        this._fireModified();
    }
    /**
     * Subscribe to modifications in the settings container.
     * @param action - Action to execute when the settings change.
     * @internal
     */
    _onModified(action) {
        if (this._modifiedEventContainer == null)
            this._modifiedEventContainer = new utils_1.EventContainer();
        this._modifiedEventContainer.subscribe(action);
    }
    /** @internal */
    _fireModified() {
        if (this._modifiedEventContainer != null)
            this._modifiedEventContainer.fire(undefined);
    }
}
exports.SettingsContainer = SettingsContainer;
